@@ -1,19 +1,19 @@
 // -*- c++ -*-
-/* $Id: main.cc 277 2006-05-10 08:05:04Z murrayc $ */
+/* $Id$ */
 
 /* Copyright (C) 2002 The gtkmm Development Team
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
@@ -23,13 +23,8 @@
 #include <glibmm/thread.h>
 #include <glibmm/wrap.h>
 #include <glibmm/iochannel.h>
-
-#include <glib/gmessages.h>
 #include <algorithm>
 
-GLIBMM_USING_STD(min)
-
-
 namespace
 {
 
@@ -288,6 +283,7 @@ SignalTimeout::SignalTimeout(GMainContext* context)
   context_ (context)
 {}
 
+/* Note that this is our equivalent of g_timeout_add(). */
 sigc::connection SignalTimeout::connect(const sigc::slot<bool>& slot,
                                         unsigned int interval, int priority)
 {
@@ -310,6 +306,41 @@ sigc::connection SignalTimeout::connect(const sigc::slot<bool>& slot,
   return connection;
 }
 
+void SignalTimeout::connect_once(const sigc::slot<void>& slot, 
+                                 unsigned int interval, int priority)
+{
+    connect(sigc::bind_return(slot, false), interval, priority);
+}
+
+/* Note that this is our equivalent of g_timeout_add_seconds(). */
+sigc::connection SignalTimeout::connect_seconds(const sigc::slot<bool>& slot,
+                                        unsigned int interval, int priority)
+{
+  SourceConnectionNode *const conn_node = new SourceConnectionNode(slot);
+  const sigc::connection connection (*conn_node->get_slot());
+
+  GSource *const source = g_timeout_source_new_seconds(interval);
+
+  if(priority != G_PRIORITY_DEFAULT)
+    g_source_set_priority(source, priority);
+
+  g_source_set_callback(
+      source, &glibmm_source_callback, conn_node,
+      &SourceConnectionNode::destroy_notify_callback);
+
+  g_source_attach(source, context_);
+  g_source_unref(source); // GMainContext holds a reference
+
+  conn_node->install(source);
+  return connection;
+}
+
+void SignalTimeout::connect_seconds_once(const sigc::slot<void>& slot, 
+                                         unsigned int interval, int priority)
+{
+    connect_seconds(sigc::bind_return(slot, false), interval, priority);
+}
+
 SignalTimeout signal_timeout()
 {
   return SignalTimeout(0); // 0 means default context
@@ -345,6 +376,11 @@ sigc::connection SignalIdle::connect(const sigc::slot<bool>& slot, int priority)
   return connection;
 }
 
+void SignalIdle::connect_once(const sigc::slot<void>& slot, int priority)
+{
+    connect(sigc::bind_return(slot, false), priority);
+}
+
 SignalIdle signal_idle()
 {
   return SignalIdle(0); // 0 means default context
